#include <stdio.h>
#include <pcap.h>
#include "decode.h"


void ProcessPacket(char *user, struct pcap_pkthdr * pkthdr, unsigned char * pkt);

void writeRandomRaw(Packet *p,struct pcap_pkthdr *hdr,char *raw);
pcap_dumper_t *mydumper;
int snaplen,datalink;

typedef void (*grinder_t)(Packet *, struct pcap_pkthdr *, u_char *); 
grinder_t grinder;

int main(int argc, char *argv[])
{
		 char errorbuf[PCAP_ERRBUF_SIZE];
		 pcap_t *readfd;
		 int pkt_cnt=0;
		 
		 if(argc!=3) {
			 fprintf(stderr,"Usage: <%s> <original trace> <new trace>\n",argv[0]);
			 exit(-1);
		 }

		 
		 readfd=pcap_open_offline(argv[1], errorbuf);
		 
		 if(readfd == NULL)
     {
        if(strstr(errorbuf, "Permission denied"))
           fprintf(stderr,"ERROR: Um... Dude.  You don't have permission to"
                      " sniff.\nTry doing this as root.\n");
        else
           fprintf(stderr,"ERROR: OpenPcap() device %s open: \n\t%s\n",argv[1],errorbuf);
		   exit(-1);
     }

		 snaplen = pcap_snapshot(readfd);
		 datalink = pcap_datalink(readfd);

		 if((mydumper=pcap_dump_open(readfd,argv[2]))==NULL) {
			 fprintf(stderr,"Error creating new trace\nExiting...\n");
        	return -1;
		 }
    
		 SetPktProcessor();
    /* Read all packets on the device.  Continue until cnt packets read */
    if(pcap_loop(readfd,pkt_cnt, (pcap_handler) ProcessPacket, NULL) < 0)
    {
        pcap_dump_close(mydumper);
    }

    pcap_dump_close(mydumper);
    return 0;                

}


void ProcessPacket(char *user, struct pcap_pkthdr * pkthdr, unsigned char * pkt)
{
	Packet p;
    (*grinder) (&p, pkthdr, pkt);
    writeRandomRaw(&p,pkthdr,pkt);
    return;
}

void writeRandomRaw(Packet *p,struct pcap_pkthdr *hdr,char *raw) {
        char *data=raw;
        int i=hdr->caplen-p->dsize+1;
        
		while(i<hdr->caplen) {
            *(data+i)=(unsigned char)(random()%255);
            i++;
        }

        pcap_dump(mydumper,hdr,data); 
        
        return;

}

int SetPktProcessor()
{
    switch(datalink)
    {
        case DLT_EN10MB:        /* Ethernet */
            grinder = DecodeEthPkt;
            break;

#ifdef DLT_IEEE802_11
    		case DLT_IEEE802_11:    
        		grinder = DecodeIEEE80211Pkt;
      			break;
#endif
        case 13:
        case DLT_IEEE802:                /* Token Ring */
            grinder = DecodeTRPkt;

            break;

        case DLT_FDDI:                /* FDDI */
            grinder = DecodeFDDIPkt;
            break;


        case DLT_SLIP:                /* Serial Line Internet Protocol */
            grinder = DecodeSlipPkt;
            break;

        case DLT_PPP:                /* point-to-point protocol */
            grinder = DecodePppPkt;
            break;
#ifdef DLT_LINUX_SLL
        case DLT_LINUX_SLL:
            grinder = DecodeLinuxSLLPkt;
            break;
#endif

#ifdef DLT_PFLOG
        case DLT_PFLOG:
            grinder = DecodePflog;
            break;
#endif

#ifdef DLT_LOOP
        case DLT_LOOP:
#endif
        case DLT_NULL:            
            grinder = DecodeNullPkt;
            break;

#ifdef DLT_RAW /* Not supported in some arch or older pcap
                * versions */
        case DLT_RAW:
            grinder = DecodeRawPkt;
            break;
#endif
#ifdef DLT_I4L_RAWIP
        case DLT_I4L_RAWIP:
            grinder = DecodeI4LRawIPPkt;
            break;
#endif

#ifdef DLT_I4L_IP
        case DLT_I4L_IP:
            if (! pv.readmode_flag && !pv.quiet_flag)
                LogMessage("Decoding I4L-ip on interface %s\n", 
                        PRINT_INTERFACE(pv.interfaces[num]));

            grinder = DecodeEthPkt;

            break;
#endif

#ifdef DLT_I4L_CISCOHDLC
        case DLT_I4L_CISCOHDLC:
            grinder = DecodeI4LCiscoIPPkt;
            break;
#endif

        default:                        /* oops, don't know how to handle this one */
            return 0;            
    
    }

    return 0;
}

